@@ -4,7 +4,18 @@
 # In[1]:
 
 
-version = '0.36'
+try:
+    from IPython.display import display, Javascript
+    display(Javascript('Jupyter.notebook.kernel.execute("nb_name = \'" + Jupyter.notebook.notebook_name + "\'");'))
+except:
+        pass
+
+
+# In[3]:
+
+
+version = '0.45'
+def is_testing(): return False
 
 import re
 import tempfile
@@ -13,17 +24,25 @@ import subprocess
 import glob
 from collections import OrderedDict
 import base64
+import shutil
+import time
+import os
+import stat
 
 
 # # Helper Functions
 
-# In[79]:
+# In[5]:
 
 
-get_ipython().run_cell_magic('javascript', '', 'var nb = IPython.notebook;\nvar kernel = IPython.notebook.kernel;\nvar command = "file_name = \'" + nb.notebook_path + "\'";\nkernel.execute(command);')
+def is_testing():
+    try:
+        return nb_name == 'lx.ipynb'
+    except:
+        return False
 
 
-# In[80]:
+# In[6]:
 
 
 def isnotebook():
@@ -34,18 +53,14 @@ def isnotebook():
     return False
 
 
-# In[81]:
+# def is_testing():
+#     try:
+#         return file_name == 'lx.ipynb'
+#     except:
+#         pass
+#     return False
 
-
-def is_testing():
-    try:
-        return file_name == 'lx.ipynb'
-    except:
-        pass
-    return False
-
-
-# In[5]:
+# In[7]:
 
 
 def testPrint(*args):
@@ -55,7 +70,7 @@ def testPrint(*args):
 
 # # Requiremtns
 
-# In[6]:
+# In[8]:
 
 
 requirements = []
@@ -81,14 +96,14 @@ from IPython.core.display import display, HTML
 
 # # Lists
 
-# In[7]:
+# In[9]:
 
 
 def listsGetPairsFromTwoLists(list1, list2):
     return list(zip(list1, list2))
 
 
-# In[8]:
+# In[10]:
 
 
 if is_testing():
@@ -98,14 +113,14 @@ if is_testing():
     print(listsGetPairsFromTwoLists(listAbcd, list1234))
 
 
-# In[9]:
+# In[11]:
 
 
 def listSequentialOverlappingPairs(l):
     return list(zip(l[:-1], l[1:]))
 
 
-# In[10]:
+# In[12]:
 
 
 if is_testing():
@@ -115,7 +130,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[11]:
+# In[13]:
 
 
 regex_cache = {}
@@ -125,33 +140,37 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[12]:
-
-
-testPrint(sListFilter(['good x', 'other good x', 'without the letter before y']))
-
-
-# In[13]:
-
-
-testPrint(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
-
-
 # In[14]:
 
 
-testPrint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
+if is_testing():
+    sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
 # In[15]:
 
 
-testPrint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
+if is_testing():
+    print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
+
+
+# In[16]:
+
+
+if is_testing():
+    print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
+
+
+# In[17]:
+
+
+if is_testing():
+    print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
 # # Text files
 
-# In[16]:
+# In[18]:
 
 
 def txtread(path):
@@ -167,7 +186,7 @@ def txtwrite(path, txt):
         return f.write(txt)
 
 
-# In[17]:
+# In[19]:
 
 
 if is_testing():
@@ -176,7 +195,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[18]:
+# In[20]:
 
 
 regex_cache = {}
@@ -188,40 +207,44 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[19]:
-
-
-testPrint(sListFilter(['good x', 'other good x', 'without the letter before y']))
-
-
-# In[20]:
-
-
-testPrint(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
-
-
 # In[21]:
 
 
-testPrint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
+if is_testing():
+    print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
 # In[22]:
 
 
-testPrint(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
+if is_testing():
+    print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
 # In[23]:
 
 
+if is_testing():
+    print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
+
+
+# In[24]:
+
+
+if is_testing():
+    print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
+
+
+# In[25]:
+
+
 if is_testing():
     print(txtread('https://arxiv.org'))
 
 
 # # Images
 
-# In[24]:
+# In[26]:
 
 
 def imread(path):
@@ -251,7 +274,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[25]:
+# In[27]:
 
 
 def imshow(array, scale=1):
@@ -266,7 +289,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[26]:
+# In[28]:
 
 
 if is_testing():
@@ -274,14 +297,14 @@ if is_testing():
     imshow(img)
 
 
-# In[27]:
+# In[29]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[28]:
+# In[30]:
 
 
 def imscaleNN(img, s):
@@ -291,7 +314,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[29]:
+# In[31]:
 
 
 if is_testing():
@@ -310,33 +333,33 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[30]:
+# In[32]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[31]:
+# In[33]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[32]:
+# In[34]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[33]:
+# In[35]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[34]:
+# In[36]:
 
 
 def imAddNoiseGauss(img, std):
@@ -348,7 +371,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[35]:
+# In[37]:
 
 
 def imJpgDegradation(img, quality):
@@ -361,7 +384,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[36]:
+# In[38]:
 
 
 def imCropCenter(img, size):
@@ -376,35 +399,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[37]:
+# In[39]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[38]:
+# In[40]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[39]:
+# In[41]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[40]:
+# In[42]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[41]:
+# In[43]:
 
 
 def imGallery(imgs, pad=0):
@@ -435,7 +458,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[42]:
+# In[44]:
 
 
 if is_testing():
@@ -443,49 +466,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[43]:
+# In[45]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[44]:
+# In[46]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[45]:
+# In[47]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[46]:
+# In[48]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[47]:
+# In[49]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[48]:
+# In[50]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[49]:
+# In[51]:
 
 
 if is_testing():
@@ -495,14 +518,14 @@ if is_testing():
 
 # # Files
 
-# In[50]:
+# In[52]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[51]:
+# In[53]:
 
 
 if is_testing():
@@ -531,9 +554,69 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
+# In[54]:
+
+
+def fiRemove(path):
+    os.remove(path)
+
+def fiRemoveDir(path):
+    shutil.rmtree(path)
+
+def fiIsEmptyDir(path):
+    return len(os.listdir(path)) == 0
+
+def fiSize(path):
+    return os.path.getsize(path)
+
+def fiSizeMb(path):
+    return os.path.getsize(path) / 2**20
+
+def fiAgeSeconds(path):
+    return time.time() - os.stat(path)[stat.ST_MTIME]
+
+def fiAgeMinutes(path):
+    return (time.time() - os.stat(path)[stat.ST_MTIME]) / 60
+
+def fiAgeHours(path):
+    return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
+
+
+# In[55]:
+
+
+if is_testing():
+    with tempfile.TemporaryDirectory() as d:
+        listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
+        
+        out_dir = os.path.join(d, "sub_dir")
+        
+        imgs_write = []
+        img = imNewBlack(100, 100)
+        img_path = os.path.join(out_dir, "1.png")
+        imwrite(img_path, img)
+        
+        if not fiIsEmptyDir(out_dir):
+            print("out_dir not empty\n")
+        print("\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
+        print("size: " + str(fiSizeMb(img_path)) + " MB")
+        print("age: " + str(fiAgeSeconds(img_path)) + 's')
+        print("age: " + str(fiAgeMinutes(img_path)) + 'min')
+        print("age: " + str(fiAgeHours(img_path)) + 'h')
+        
+        print('\nrm 1.png')
+        fiRemove(os.path.join(d, "sub_dir", "1.png"))
+        print("files:\n" + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
+        
+        if fiIsEmptyDir(out_dir):
+            print("\nRemove empty dir")
+            fiRemoveDir(out_dir)
+        print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
+
+
 # # Jupyter
 
-# In[52]:
+# In[56]:
 
 
 def jnImageRenderingPixelated():
@@ -545,7 +628,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[53]:
+# In[57]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -558,7 +641,7 @@ if is_testing():
     
 
 
-# In[54]:
+# In[58]:
 
 
 def jnImgToHtmlB64(img):
@@ -582,7 +665,7 @@ def jnImageRow(imgs, names=None, width=100):
     display(HTML(html))
 
 
-# In[55]:
+# In[59]:
 
 
 if is_testing():
@@ -590,7 +673,7 @@ if is_testing():
    
 
 
-# In[56]:
+# In[60]:
 
 
 
@@ -600,7 +683,7 @@ if is_testing():
 
 # # Shell
 
-# In[82]:
+# In[61]:
 
 
 def shell(cmd):
@@ -615,43 +698,47 @@ def shell(cmd):
 
 # # GIT
 
-# In[58]:
+# In[62]:
 
 
 def gitGetHash():
     return shell('git rev-parse HEAD').strip()
 
-testPrint(gitGetHash())
+if is_testing():
+    print(gitGetHash())
 
 
-# In[59]:
+# In[63]:
 
 
 def gitGetBranch():
     return shell('git rev-parse --abbrev-ref HEAD').strip()
 
-testPrint(gitGetBranch())
+if is_testing():
+    print(gitGetBranch())
 
 
-# In[60]:
+# In[64]:
 
 
 def gitGetMessage():
     return shell('git log --format=%B -n 1 HEAD').strip()
 
-testPrint(gitGetMessage())
+if is_testing():
+    print(gitGetMessage())
 
 
-# In[61]:
+# In[65]:
 
 
 def gitGetAllFiles():
     return shell('git ls-tree -r --name-only HEAD').split("\n")
 
-testPrint(gitGetAllFiles())
+if is_testing():
+    print(gitGetAllFiles())
 
 
-# In[62]:
+# In[66]:
 
 
 def gitListByTime(path=None):
@@ -663,16 +750,18 @@ def gitListByTime(path=None):
     
     return [path for _, path in sorted(zip(times, paths))]
     
-testPrint(gitListByTime()[-3:])
+if is_testing():
+    print(gitListByTime()[-3:])
 
 
-# In[84]:
+# In[67]:
 
 
 def shellSortDirsBySize():
     return shell('du -h | sort -h')
 
-testPrint(shellSortDirsBySize())
+if is_testing():
+    print(shellSortDirsBySize())
 
 
 # # Package Files
@@ -680,11 +769,17 @@ testPrint(shellSortDirsBySize())
 # In[ ]:
 
 
+
+
+
+# In[68]:
+
+
 if is_testing():
     package_files = {}
 
 
-# In[ ]:
+# In[69]:
 
 
 def shell(cmd):
@@ -692,7 +787,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[ ]:
+# In[70]:
 
 
 if is_testing():
@@ -724,7 +819,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[ ]:
+# In[71]:
 
 
 if is_testing():
@@ -734,7 +829,7 @@ description-file = README.md
 """
 
 
-# In[ ]:
+# In[72]:
 
 
 if is_testing():
@@ -757,14 +852,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[ ]:
+# In[73]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[ ]:
+# In[74]:
 
 
 if is_testing():
@@ -778,7 +873,7 @@ if is_testing():
 
 
 
-# In[ ]:
+# In[75]:
 
 
 if is_testing():
@@ -789,7 +884,7 @@ lx/__init__.py
 """
 
 
-# In[ ]:
+# In[76]:
 
 
 if is_testing():
@@ -814,21 +909,3 @@ if is_testing():
 
 
 
-
-# In[ ]:
-
-
-
-
-
-# In[ ]:
-
-
-
-
-
-# In[ ]:
-
-
-
-
